from cffi import FFI

import sys

def build(include_dirs, library_dirs, libraries):

  ffibuilder = FFI()
  ffibuilder.set_source("__c_gqlite", "#include <gqlite-c.h>", include_dirs = include_dirs, libraries=libraries, library_dirs=library_dirs)
  ffibuilder.cdef(
  """

  typedef struct gqlite_api_context* gqlite_api_context_t;
  typedef struct gqlite_connection* gqlite_connection_t;
  typedef struct gqlite_backend* gqlite_backend_t;
  typedef struct gqlite_value* gqlite_value_t;

  gqlite_api_context_t gqlite_api_context_create();
  void gqlite_api_context_destroy(gqlite_api_context_t);
  const char* gqlite_api_context_get_message(gqlite_api_context_t);
  bool gqlite_api_context_has_error(gqlite_api_context_t);
  void gqlite_api_context_clear_error(gqlite_api_context_t);
  gqlite_connection_t gqlite_connection_create_from_sqlite(gqlite_api_context_t, void*, gqlite_value_t _options);
  gqlite_connection_t gqlite_connection_create_from_sqlite_file(gqlite_api_context_t, const char*, gqlite_value_t _options);
  void gqlite_connection_destroy(gqlite_api_context_t, gqlite_connection_t);
  gqlite_value_t gqlite_connection_oc_query(gqlite_api_context_t, gqlite_connection_t, const char*, gqlite_value_t);
  gqlite_value_t gqlite_value_create(gqlite_api_context_t);
  void gqlite_value_destroy(gqlite_api_context_t, gqlite_value_t);
  const char* gqlite_value_to_json(gqlite_api_context_t, gqlite_value_t);
  bool gqlite_value_is_valid(gqlite_api_context_t, gqlite_value_t);

  """)

  return ffibuilder

def build_for_pip():
  return build(["src/"], ["build"], ["gqlite", "stdc++", "sqlite3"])

if __name__ == "__main__":
  build([sys.argv[1] + "/include"], ["."], ["gqlite"]).compile(verbose=True)


import __c_gqlite as capi
import json

class Error(Exception):
  def __init__(self, msg):
    self.msg = msg

api_context = capi.lib.gqlite_api_context_create()

class Connection(object):
  def __init__(self, sqlite_filename = None) -> None:
    self.empty_value = self.__call_capi(capi.lib.gqlite_value_create)
    if sqlite_filename != None:
      self.handle = self.__call_capi(capi.lib.gqlite_connection_create_from_sqlite_file, sqlite_filename.encode('utf-8'), capi.ffi.NULL  )
    else:
      raise Error("No connection backend was selected.")
  def execute_oc_query(self, query, bindings = None):
    ret = self.__call_capi(capi.lib.gqlite_connection_oc_query, self.handle, query.encode('utf-8'), capi.ffi.NULL)
    if self.__call_capi(capi.lib.gqlite_value_is_valid, ret):
      val = json.loads(capi.ffi.string(self.__call_capi(capi.lib.gqlite_value_to_json, ret)))
    else:
      return None
    self.__call_capi(capi.lib.gqlite_value_destroy, ret)
    return val

  def __call_capi(self, name, *args):
    ret = name(api_context, *args)
    if capi.lib.gqlite_api_context_has_error(api_context):
      err = capi.ffi.string(capi.lib.gqlite_api_context_get_message(api_context)).decode('utf-8')
      capi.lib.gqlite_api_context_clear_error(api_context)
      raise Error(err)
    return ret

def connect(*args):
  return Connection(*args)


#!/usr/bin/env python

import os
import sys

from setuptools import setup, find_packages, Extension
from setuptools.command.build_clib import build_clib
from distutils.ccompiler import new_compiler

setup(
    name="gqlitedb",
    version="1.0.0",
    description="Python bindings for GQLite, a Graph Query library",
    long_description=open("README.md", "rt").read(),
    url="https://gitlab.com/gqlite/gqlite",
    author="Cyrille Berger",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    packages=find_packages(),
    libraries = [ ('gqlite', {'sources': ['src/gqlite-amalgamate.cpp'], 'cflags': ['-std=c++20', '-fPIC', '-Isrc'], 'build-clib': 'build/'})],
    cmdclass = {'build_clib': build_clib},
    cffi_modules=[
        "./build-ffi.py:build_for_pip",
    ]
)